



gr_filter_design --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
gr_plot --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
gr_plot_const --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
gr_plot_fft --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
gr_plot_iq --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
gr_plot_psd --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
start gr_filter_design && ping -n 30 127.0.0.1 >nul && taskkill /im gr_filter_design.exe /f
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
